@@ -9,12 +9,17 @@
 #' `MsBackendMetaboLights` retrieves and represents mass spectrometry (MS)
 #' data from metabolomics experiments stored in the
 #' [MetaboLights](https://www.ebi.ac.uk/metabolights/) repository. The backend
-#' directly extends the [MsBackendMzR] backend from the *Spectra* package and
-#' hence supports MS data in mzML, netCDF and mzXML format. Data in other
-#' formats can not be loaded with `MsBackendMetaboLights`. Upon initialization
-#' with the `backendInitialize()` method, the `MsBackendMetaboLights` backend
-#' downloads and caches the MS data files of an experiment locally avoiding
-#' hence repeated download of the data.
+#' directly extends the [Spectra::MsBackendMzR] backend from the *Spectra*
+#' package and hence supports MS data in mzML, netCDF and mzXML format. Data
+#' in other formats can not be loaded with `MsBackendMetaboLights`.
+#' Upon initialization with the `backendInitialize()` method, the
+#' `MsBackendMetaboLights` backend downloads and caches the MS data files of
+#' an experiment locally avoiding hence repeated download of the data.
+#' The local data cache is managed by Bioconductor's *BiocFileCache* package.
+#' See the help and vignettes from that package for details on cached data
+#' resources. Additional utility function for management of cached files are
+#' also provided by *MsBackendMetaboLights*. See help for
+#' [mtbls_cached_data_files()] for more information.
 #'
 #' @section Initialization and loading of data:
 #'
@@ -45,6 +50,17 @@
 #' will check the local cache and eventually download missing data files from
 #' the MetaboLights repository.
 #'
+#' @note
+#'
+#' To account for high server load and eventually failing or rejected
+#' downloads from the MetaboLights ftp server, the download functions
+#' repeatedly retry to download a file. An error is thrown if download fails
+#' for 3 consecutive attempts. Between each attemp, the function waits
+#' for an increasing time period (5 seconds between the first and second
+#' and 10 seconds between the 2nd and 3rd attempt). This time period can
+#' also be configured with the `"metabolights.sleep_mult"` option, which
+#' defines the *sleep time multiplicator* (defaults to 5).
+#'
 #' @param object an instance of `MsBackendMetaboLights`.
 #'
 #' @param mtblsId `character(1)` with the ID of a single MetaboLights data
@@ -106,10 +122,6 @@
 #' The `MsBackendMetaboLights` backend is considered *read-only* and does
 #' thus not support changing *m/z* and intensity values directly.
 #'
-#' Also, merging of MS data of `MsBackendMetaboLights` is not supported and
-#' thus `c()` of several `Spectra` with MS data represented by
-#' `MsBackendMetaboLights` will throw an error.
-#'
 #' @importClassesFrom Spectra MsBackendMzR
 #'
 #' @importClassesFrom Spectra MsBackendDataFrame
@@ -194,19 +206,6 @@ setMethod(
         object <- as(object, "MsBackendMetaboLights")
     })
 
-#' @rdname MsBackendMetaboLights
-#'
-#' @importFrom ProtGenerics backendMerge
-#'
-#' @exportMethod backendMerge
-setMethod(
-    "backendMerge", "MsBackendMetaboLights",
-    function(object, ...) {
-        stop("Merging of backends of type 'MsBackendMetaboLights' is not ",
-             "supported. Use 'setBackend()' to change to a backend that ",
-             "supports merging, such as the 'MsBackendMemory'.")
-    })
-
 #' @rdname MsBackendMetaboLights
 #'
 #' @importMethodsFrom Spectra backendRequiredSpectraVariables
@@ -334,6 +333,11 @@ mtbls_sync <- function(x, offline = FALSE) {
 #'   Parameter `fileName` allows to specify names of selected data files to
 #'   sync.
 #'
+#' - `mtbls_delete_cache()`: removes all local content for the MetaboLights
+#'   data set with ID `mtblsId`. This will delete eventually present
+#'   locally cached data files for the specified data set. This does not
+#'   change any other data eventually present in the local `BiocFileCache`.
+#'
 #' @param x `character(1)` with the ID of the MetaboLights data set (usually
 #'     starting with a *MTBLS* followed by a number).
 #'
@@ -419,13 +423,15 @@ mtbls_list_files <- function(x = character(), pattern = NULL) {
     cu <- new_handle()
     handle_setopt(cu, ftp_use_epsv = TRUE, dirlistonly = TRUE)
     tryCatch({
-        con <- curl(url = mtbls_ftp_path(x, mustWork = FALSE), "r", handle = cu)
+        con <- .retry(
+            curl(url = mtbls_ftp_path(x, mustWork = FALSE), "r", handle = cu),
+            sleep_mult = .sleep_mult())
     }, error = function(e) {
         stop("Failed to connect to MetaboLights. No internet connection? ",
              "Does the data set \"", x, "\" exist?\n - ", e$message,
              call. = FALSE)
     })
-    fls <- readLines(con)
+    fls <- .retry(readLines(con), sleep_mult = .sleep_mult())
     close(con)
     if (length(pattern))
         fls[grepl(pattern, fls)]
@@ -446,9 +452,10 @@ mtbls_list_files <- function(x = character(), pattern = NULL) {
     fpath <- mtbls_ftp_path(x, mustWork = FALSE)
     a_fls <- mtbls_list_files(x, pattern = "^a_")
     res <- lapply(a_fls, function(z) {
-        read.table(paste0(fpath, z),
-                   sep = "\t", header = TRUE,
-                   check.names = FALSE)
+        .retry(read.table(paste0(fpath, z),
+                          sep = "\t", header = TRUE,
+                          check.names = FALSE),
+               sleep_mult = .sleep_mult())
     })
     names(res) <- a_fls
     res
@@ -534,10 +541,20 @@ mtbls_cached_data_files <- function(mtblsId = character(),
 #'   file/table
 #' - `"rpath"`: the name of the cached data file (full local path)
 #'
+#' @note
+#'
+#' Download from MsBackendMetaboLights is tried 3 times with an increasing time
+#' delay between tries that can be configured using the
+#' `"metabolights.sleep_mult"` option.
+#'
 #' @importFrom BiocFileCache BiocFileCache
 #'
+#' @importFrom progress progress_bar
+#'
 #' @importMethodsFrom BiocFileCache bfcrpath bfcmeta<-
 #'
+#' @importFrom utils capture.output
+#'
 #' @noRd
 .mtbls_data_files <- function(mtblsId = character(), assayName = character(),
                               pattern = "mzML$|CDF$|mzXML$",
@@ -576,7 +593,19 @@ mtbls_cached_data_files <- function(mtblsId = character(),
     }
     ## Cache files
     bfc <- BiocFileCache()
-    lfiles <- bfcrpath(bfc, paste0(fpath, ffiles), fname = "exact")
+    pb <- progress_bar$new(format = paste0("[:bar] :current/:",
+                                           "total (:percent) in ",
+                                           ":elapsed"),
+                           total = length(ffiles), clear = FALSE)
+    lfiles <- unlist(lapply(ffiles, function(z) {
+        pb$tick()
+        invisible(capture.output(suppressMessages(
+            f <- .retry(bfcrpath(bfc, paste0(fpath, z), fname = "exact"),
+                        sleep_mult = .sleep_mult()))))
+        f
+    }))
+
+    ## lfiles <- bfcrpath(bfc, paste0(fpath, ffiles), fname = "exact")
     ## Add and store metadata to the cached files
     mdata <- data.frame(
         rid = names(lfiles),
@@ -596,14 +625,14 @@ mtbls_cached_data_files <- function(mtblsId = character(),
 #' returned `data.frame` has the same format as the one returned by
 #' `.mtbls_data_files()`.
 #'
-#' @importMethodsFrom BiocFileCache bfcmetalist bfcquery
+#' @importMethodsFrom BiocFileCache bfcquery
 #'
 #' @noRd
 .mtbls_data_files_offline <- function(mtblsId = character(),
                                       assayName = character(),
                                       pattern = "mzML$|CDF$|mzXML$") {
     bfc <- BiocFileCache()
-    if (!any(bfcmetalist(bfc) == "MTBLS"))
+    if (!.mtbls_has_mtbls_table())
         stop("No local MetaboLights cache available. Please re-run with ",
              "'offline = FALSE' first.", call. = FALSE)
     res <- as.data.frame(bfcquery(bfc, mtblsId, field = "mtbls_id"))
@@ -617,3 +646,27 @@ mtbls_cached_data_files <- function(mtblsId = character(),
     res[, c("rid", "mtbls_id", "mtbls_assay_name",
             "derived_spectral_data_file", "rpath")]
 }
+
+#' @importMethodsFrom BiocFileCache bfcmetalist
+#'
+#' @noRd
+.mtbls_has_mtbls_table <- function() {
+    bfc <- BiocFileCache()
+    any(bfcmetalist(bfc) == "MTBLS")
+}
+
+#' @rdname MetaboLights-utils
+#'
+#' @importFrom BiocFileCache bfcremove bfcinfo
+#'
+#' @export
+mtbls_delete_cache <- function(mtblsId = character()) {
+    bfc <- BiocFileCache()
+    b <- as.data.frame(bfcinfo(bfc))
+    if (nrow(b) && any(colnames(b) == "mtbls_id")) {
+        if (length(mtblsId)) {
+            rem <- b[b$mtbls_id %in% mtblsId, ]
+            bfcremove(bfc, rids = rem$rid)
+        }
+    }
+}
